#include "hmapfile.h"
#include "qstring.h"
#include "qapplication.h"
#include <iostream>
using namespace std;

HMapFile::HMapFile(QFile *file, QPtrList<LowQAFile> *llqaFiles)
{
    f = file;
    llqaf = llqaFiles;
    nFiles = llqaf->count();
    doc = new QDomDocument("hhistorymap");
    headerDone = FALSE;
}

HMapFile::~HMapFile(void)
{
    delete doc;
}

void HMapFile::createHeader(QString *inDir, QString *pat)
{
    QDomComment com;
    
    rootElem = doc->createElement("mapfile");
    doc->appendChild(rootElem);
    com.setData("Meaning of the histogram status variables: 0 = GOOD, 1 = WARN and 2 = BAD");
    rootElem.appendChild(com);
    header = doc->createElement("header");
    rootElem.appendChild(header);
    ver = doc->createElement("version");
    header.appendChild(ver);
    t = doc->createTextNode(QString::number(HHISTORY_VERSION));
    ver.appendChild(t);
    dir = doc->createElement("directory");
    header.appendChild(dir);
    tdir = doc->createTextNode(*inDir);
    dir.appendChild(tdir);
    mask = doc->createElement("filepattern");
    header.appendChild(mask);
    tmask = doc->createTextNode(*pat);
    mask.appendChild(tmask);
    headerDone = TRUE;
}

bool HMapFile::createBody(void)
{
    QDomNode t;
    if(headerDone)
    {
	body = doc->createElement("filestatus");
	rootElem.appendChild(body);
	for(int i=0;i<nFiles;i++)
	{
	    t = llqaf->at(i)->getStatusXml(doc);
	    body.appendChild(t);
	}
	return TRUE;
    }
    return FALSE;
}

void HMapFile::writeFile(void)
{
    QString xml;
    
    xml = doc->toString();
    f->writeBlock(xml.latin1(),xml.length());
}

bool HMapFile::readDocument(QString *indir, QString *pat, QProgressBar *bar)
{
    QDomNode node;
    QString *msg;
    int line;
    int nFiles;
    
    msg = new QString();
    if(doc->setContent(f,msg,&line))
    {
	rootElem = doc->documentElement();
	nFiles = doc->elementsByTagName("file").count();
	if(nFiles > 0)
	    bar->setTotalSteps(nFiles);
	node = rootElem.firstChild();
	if(!node.isNull())
	{
//	    if(node.isComment())
//	    {
//		node = node.toElement().firstChild();
//	    }
	    if(!node.isNull())
	    {
		header = node.toElement();
		if(header.tagName().localeAwareCompare("header") ==0)
		{
		    node = header.firstChild();
		    if(!node.isNull())
		    {
			ver = node.toElement();
			if((ver.tagName().localeAwareCompare("version") == 0) &&
			   (ver.text().toInt() == HHISTORY_VERSION))
			{
			    node = ver.nextSibling();
			    if(!node.isNull())
			    {
				dir = node.toElement();
				if(dir.tagName().localeAwareCompare("directory") == 0)
				{
				    if(dir.text().localeAwareCompare(*indir) == 0)
				    {
					node = dir.nextSibling();
					if(node.isElement())
					{
					    mask = node.toElement();
					    if(mask.tagName().localeAwareCompare("filepattern") == 0)
					    {
						if(mask.text().localeAwareCompare(*pat) == 0)
						{
						    if(loadFileList(bar))
							return TRUE;
						    else
							return FALSE;
						}
						else
						    return FALSE;
					    }
					}
				    }
				    else
					return FALSE;
				}
			    }
			}
		    }
		}
	    }
	}
    }
    else
    {
	cout << msg->latin1() << " at line "<< line << endl;
    }
    return FALSE;
}


bool HMapFile::loadFileList(QProgressBar *bar)
{
    QDomElement e, e1, e2;
    QDomNode node;
    QString fName;
    QString hName;
    int hStatus;
    QPtrListIterator<LowQAFile> it(*llqaf);
    LowQAFile *lf;
    int prog = 0;
    
    bar->setProgress(prog);
    node = header.nextSibling();
    if(!node.isNull())
    {
	e = node.toElement();
	if(e.tagName().localeAwareCompare("filestatus") == 0)
	{
	    node = e.firstChild();
	    while(!node.isNull())
	    {
		e1 = node.toElement();
		if(e1.tagName().localeAwareCompare("file") == 0)
		{
		    prog++;
		    bar->setProgress(prog);
		    fName = e1.attribute("name");
		    it.toFirst();
		    while((lf = it.current()) != NULL)
		    {
			++it;
			if(lf->getFileName()->localeAwareCompare(fName) == 0)
			{
			    qApp->processEvents(3);
			    lf->readHistStatus(e1);
			}
		    }
		}
		node = e1.nextSibling();
	    }
	}
    }
    return TRUE;
}
