/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/


void HHistory::init()
{
    bool ok;
    QDir *dir;
    
    settings = new QSettings();
    settings->insertSearchPath( QSettings::Unix, "$HOME/.qt/");
    settings->beginGroup("/hhistory");
    settings->beginGroup("/input");
    inDir = new QString(settings->readEntry("/directory","./").latin1());
    lineEdit1->setText(inDir->latin1());
    filePattern = new QString(settings->readEntry("/filepattern","*.root").latin1());
    lineEdit2->setText(filePattern->latin1());
    mapFile = new QString(settings->readEntry("/mapfile","root.map").latin1());
    lineEdit3->setText(mapFile->latin1());
    settings->endGroup(); /* /input */
    settings->beginGroup("/defaults");
    delay = settings->readNumEntry("/delay",2,&ok);
    spinBox1->setValue(delay);
    
    asymquality = settings->readBoolEntry("/qualityasym",FALSE);
    qualityLowGood = settings->readDoubleEntry("/qualitylowgood",0.5);
    qualityLowWarn = settings->readDoubleEntry("/qualitylowwarn",0.5);
    qualityHighGood = settings->readDoubleEntry("/qualityhighgood",1.0);
    qualityHighWarn = settings->readDoubleEntry("/qualityhighwarn",1.0);
    
    if(asymquality)
    {
	textLabel4->setText(QString("-%1 > good < %2").arg(qualityLowGood).arg(qualityHighGood));
	textLabel5->setText(QString(" -%1 > bad < %2").arg(qualityLowWarn).arg(qualityHighWarn));
    }
    else
    {
	textLabel4->setText(QString("- %1 > good < %2").arg(qualityHighGood).arg(qualityHighGood));
	textLabel5->setText(QString("- %1 > bad < %2").arg(qualityHighWarn).arg(qualityHighWarn));
    }
    cutLow = settings->readDoubleEntry("/cutlow",0.0);
    cutHigh = settings->readDoubleEntry("/cuthigh",0.999);
    textLabel8_2->setText(QString("%1 < xVal < %2").arg(cutLow).arg(cutHigh));
    settings->endGroup(); /* /defaults*/
    settings->beginGroup("/help");
    helpName = new QString(settings->readEntry("/helpfile","./"));
    if(helpName->startsWith("./"))
    {
	dir = new QDir(helpName->latin1());
	if(!helpName->endsWith("/"))
	{
	    helpName->remove(0,helpName->findRev("/",-1));
	}
	helpName->prepend(dir->absPath());	
	delete dir;
    }
    settings->endGroup(); /* /help */
    settings->endGroup(); /* /hhistory */
    
    connect(this,SIGNAL(updateFileList(void)),this,SLOT(rebuildFileList(void)));
    connect(this, SIGNAL(updateHistogramViewer(void)), this, SLOT(showNewHistogram(void)));
    connect(this, SIGNAL(updateCutHists(void)),this, SLOT(newCutHists(void)));
    
    table2->setColumnReadOnly(0,kTRUE);
    table2->setColumnReadOnly(1,kTRUE);
    table2->horizontalHeader()->setLabel(0,"Status");
    table2->horizontalHeader()->setLabel(1,"File name");
    listView2->clear();
    actualFile = 0;
    fileTimer=NULL;
    pause =FALSE;
    pushButton16->setEnabled(FALSE);
    pushButton17->setEnabled(FALSE);
    files.setAutoDelete(TRUE);
    meanHistogram = NULL;
    phist = NULL;
    meanValueLine = NULL;
    hMeanPlus_1Sigma = NULL;
    hMeanMinus_1Sigma = NULL;
    hMeanPlus_2Sigma = NULL;
    hMeanMinus_2Sigma = NULL;
    grG = NULL;
    grW = NULL;
    grB = NULL;
    histStatus = NULL;
    histogram = new QString("test");
    path = new QString("/");
    Canvas2->getCanvas()->SetFrameFillColor(196);
}


void HHistory::destroy()
{
    if(settings)
    {
	settings->beginGroup("/hhistory");
	settings->beginGroup("/input");
	settings->writeEntry("/directory",*inDir);
	settings->writeEntry("/filepattern",*filePattern);
	settings->writeEntry("/mapfile",*mapFile);
	settings->endGroup(); /* /input*/
	settings->beginGroup("/defaults");
	settings->writeEntry("/delay",delay);
	settings->writeEntry("/qualityasym",asymquality);
	settings->writeEntry("/qualitylowgood",(double)qualityLowGood);
	settings->writeEntry("/qualitylowwarn",(double)qualityLowWarn);
	settings->writeEntry("/qualityhighgood",(double)qualityHighGood);
	settings->writeEntry("/qualityhighwarn",(double)qualityHighWarn);
	settings->writeEntry("/cutlow",(double)cutLow);
	settings->writeEntry("/cuthigh",(double)cutHigh);
	settings->endGroup(); /* /defaults */
	settings->beginGroup("/help");
	settings->writeEntry("/helpfile",*helpName);
	settings->endGroup(); /* help */
	settings->endGroup(); /* /hhistory */
	delete settings;
    }
}


void HHistory::selectInputDir( void )
{
    QString tmp;
    tmp =QFileDialog::getExistingDirectory("",this,"InputDir","Select input directory");
    inDir->truncate(0);
    inDir->append(tmp.latin1());
    lineEdit1->setText(inDir->latin1());
}


void HHistory::selectMapFile( void )
{
    QString tmp;
    tmp =QFileDialog::getOpenFileName("","*.map",this,"InputDir","Select input directory");
    mapFile->truncate(0);
    mapFile->append(tmp.latin1());
    lineEdit3->setText(inDir->latin1());
}


void HHistory::newDelay( int val )
{
    delay = val;
}


void HHistory::setQualityRange( void )
{
    QualityRange *dlg;
    
    dlg = new QualityRange(this, "qarange");
    connect(dlg,SIGNAL(qualityRange(float,float,float,float,bool)),this, SLOT(newQuality(float, float, float, float,bool)));
    dlg->initValues(qualityLowGood, qualityLowWarn, qualityHighGood, qualityHighWarn, asymquality);
    if(dlg->exec()==QDialog::Accepted)
    {
	if(asymquality)
	{
	    textLabel4->setText(QString("- %1 > good < %2").arg(qualityLowGood).arg(qualityHighGood));
	    textLabel5->setText(QString(" - %1 < bad > %2").arg(qualityLowWarn).arg(qualityHighWarn));
	}
	else
	{
	    textLabel4->setText(QString("- %1 > good < %2").arg(qualityHighGood).arg(qualityHighGood));
	    textLabel5->setText(QString("- %1 > bad < %2").arg(qualityHighWarn).arg(qualityHighWarn));
	}
    }
    delete dlg;
}


void HHistory::newQuality( float lowGood, float lowWarn, float highGood, float highWarn, bool asym )
{
    qualityLowGood = lowGood;
    qualityLowWarn = lowWarn;
    qualityHighGood = highGood;
    qualityHighWarn = highWarn;
    asymquality = asym;
    if(asymquality)
    {
	if(qualityLowGood > qualityLowWarn)
	{
	    qualityLowGood = qualityLowWarn;
	}
	if(qualityHighGood > qualityHighWarn)
	{
	    qualityHighGood = qualityHighWarn;
	}
    }
    else
    {
	if(qualityHighGood > qualityHighWarn)
	{
	    qualityHighGood = qualityHighWarn;
	}
    }
    if(meanHistogram)
	emit updateCutHists();
}


void HHistory::newHistRange( float low, float high )
{
    cutLow = low;
    cutHigh = high;
}


void HHistory::newHistCuts( void )
{
    HistRange *dlg;
    
    dlg = new HistRange(this,"hrange");
    connect(dlg,SIGNAL(newHistevalRange(float,float)),this,SLOT(newHistRange(float,float)));
    dlg->initValues(cutLow,cutHigh);
    if(dlg->exec()==QDialog::Accepted)
    {
	textLabel8_2->setText(QString("%1 < xVal < %2").arg(cutLow).arg(cutHigh)); 
    }
 }


void HHistory::loadFiles( void )
{
    QDir *dir;
    QStringList::iterator it;
    LowQAFile *llqa;
    TFile *f;
    QListViewItem *rootElement;
    int n=0;
    
    files.clear();
    listView2->clear();
    dir = new QDir(*inDir, *filePattern);
    if(dir->exists())
    {
	fileList=NULL;
	fileList = dir->entryList(*filePattern);
	if(fileList.count() > 0)
	{
	    progressBar1->setTotalSteps(fileList.count()+10);
	    progressBar1->setProgress(0);
	    textLabel1->setText("Scanning files...");
	    table2->setNumRows(fileList.count());
	    table2->setColumnWidth(1,500);
	    table2->setColumnWidth(0,75);
	    for(it = fileList.begin();it != fileList.end();++it)
	    {
		qApp->processEvents(3);
		if(QFile::exists(((*inDir)+"/"+(*it)).latin1()))
		{
		    llqa = new LowQAFile(((*inDir)+"/"+(*it)));
		    files.append(llqa);
		    n++;
		    progressBar1->setProgress(n);
		}
	    }   
	}
	nFiles = fileList.count();
	emit updateFileList();
	rootElement = new QListViewItem(listView2,"Root File");
	f = new TFile(files.first()->getFileName()->latin1(),"READ");
	f->cd();
	listDirectory(rootElement);
	listView2->setOpen(rootElement,TRUE);
	progressBar1->setProgress(fileList.count()+10);
	textLabel1->setText("Done scanning files!");
	fileTimer = new QTimer(this,"prog");
	connect(fileTimer, SIGNAL(timeout(void)),this,SLOT(hideProgress(void)));
	fileTimer->start(1000,TRUE);
	delete f;
   }
}


void HHistory::newInDir( void )
{
    inDir->truncate(0);
    inDir->append(lineEdit1->text());
}


void HHistory::newfilePattern( void )
{
    filePattern->truncate(0);
    filePattern->append(lineEdit2->text());
}

void HHistory::newMapName( void )
{
    mapFile->truncate(0);
    mapFile->append(lineEdit3->text());
}


void HHistory::loadMapFile( void )
{
    QFile *map;
    HMapFile *doc;
    
    if(QFile::exists(*mapFile))
    {
	textLabel1->setText("Reading Map file...");
	map = new QFile(*mapFile);
	if(map->open(IO_ReadOnly))
	{
	    doc = new HMapFile(map, &files);
	    if(doc->readDocument(inDir, filePattern, progressBar1))
	    {
		emit updateFileList();
	    }
	    map->close();
	    delete doc;
	    fileTimer = new QTimer(this,"prog");
	    connect(fileTimer, SIGNAL(timeout(void)),this,SLOT(hideProgress(void)));
	    fileTimer->start(1000,TRUE);
	}
	delete map;
	textLabel1->setText("");
    }
    else
	QMessageBox::critical(this,"Error locating map file",tr("Could not access map file %1").arg(mapFile->latin1()));
}


void HHistory::saveMapFile( void )
{
    QFile *map;
    HMapFile *doc;
    QString xml;
    
    if(QFile::exists(*mapFile))
    {
	QFile::remove(*mapFile);
    }
    map = new QFile(*mapFile);
    if(map->open(IO_WriteOnly))
    {
	    doc = new HMapFile(map, &files);
	    doc->createHeader(inDir, filePattern);
	    if(doc->createBody())
		doc->writeFile();
	    delete doc;
    }
    delete map;
}


void HHistory::rebuildFileList( void )
{
    int i =0;
  
    LowQAFile *file;
    for(file = files.first(); file; file=files.next())
    {
	table2->setText(i,1,*file->getFileName());
	table2->setText(i,0,file->getStatus((*path+*histogram))==LowQAFile::GOOD?"GOOD":file->getStatus((*path+*histogram))==LowQAFile::WARN?"WARN":"BAD");
	i++;
    }
}


void HHistory::listDirectory( QListViewItem *root )
{
    TList *dir;
    TIterator *iter;
    TKey *key;
    QListViewItem *element;
    
   dir = gDirectory->GetListOfKeys();
    if(dir)
    {
	iter = dir->MakeIterator();
	while((key = (TKey *)iter->Next()) != NULL)
	{
	    element = new QListViewItem(root, key->GetName());
	    if(key->IsFolder())
	    {
		gDirectory->cd(key->GetName());
		listDirectory(element);
	    }
	}
	gDirectory->cd("..");
    }
}


void HHistory::newHistogramSelected( QListViewItem *item )
{
    HistSelectDlg *dlg;
    
    path->truncate(0);;
    if(item->childCount() ==0)
    {
	histogram->truncate(0);
	histogram->append(item->text(0));
	while(item->parent() != 0)
	{
	    if((item->parent())->parent() == 0)
	    {
		path->prepend("/");
	    }
	    else
	    {
		path->prepend(item->parent()->text(0)+"/");
	    }
	    item = item->parent();
	}
	actualFile=0;
	dlg = new HistSelectDlg(this,"hselect");
	connect(dlg,SIGNAL(newHistogramType(int)),this, SLOT(newHistogramType(int)));
	dlg->exec();
	emit updateHistogramViewer();
	Canvas2->Clear();
	pushButton10->setEnabled(FALSE);
	pushButton11->setEnabled(FALSE);
	if(meanHistogram)
	    delete meanHistogram;
	meanHistogram = NULL;
   }   
}


void HHistory::showNewHistogram( void )
{
    if(phist)
	delete phist;
   Canvas1->cd();
    phist = files.at(actualFile)->getQAHistogram(histogram,path,FALSE);
    if(phist)
	phist->Draw();
    Canvas1->Update();
}


void HHistory::startFileLoop( void )
{
    if(!fileTimer)
    {
	fileTimer = new QTimer(this,"fileLoopTimer");
	connect(fileTimer, SIGNAL(timeout(void)), this, SLOT(nextFile(void)));
    }
    fileTimer->start(delay*1000);
    pushButton16->setEnabled(TRUE);
    pushButton17->setEnabled(TRUE);
}


void HHistory::nextFile( void )
{
    if(actualFile <( files.count()+1))
    {
	actualFile++;
	emit updateHistogramViewer();
    }
    else
    {
	actualFile=0;
	fileTimer->stop();
    }
}


void HHistory::stopFileLoop( void )
{
    fileTimer->stop();
    pushButton16->setEnabled(FALSE);
    pushButton17->setEnabled(FALSE);
}


void HHistory::pauseFileLoop( void )
{
    if(!pause)
    {
	fileTimer->stop();
	pause = TRUE;
	pushButton16->setText("Continue");
    }
    else
    {
	fileTimer->start(delay*1000);
	pause=FALSE;
	pushButton16->setText("Pause");
    }
}


void HHistory::nextHistogram( void )
{
    if(actualFile < (files.count() + 1))
    {
	actualFile++;
	emit updateHistogramViewer();
    }
}

void HHistory::previouseHistogram( void )
{
    if(actualFile > 0)
    {
	actualFile--;
	emit updateHistogramViewer();
    }
}


void HHistory::scanFiles( void )
{
   Canvas2->cd();
   meanHistogram = new TH1F("mean","Mean",nFiles,0,nFiles);
   meanHistogram->SetXTitle("File number");
   meanHistogram->SetYTitle("Hi/File mean value");
   meanHistogram->SetStats(FALSE);
   meanHistogram->GetYaxis()->SetTitleOffset(1.2);
   pd = new QProgressDialog(tr("Scanning thru %1 files...").arg(nFiles),"Cancel",files.count(),this,"progDlg");
   connect(pd,SIGNAL(cancelled(void)),this, SLOT(stopScan(void)));
   doScan = TRUE;
    pd->setProgress(0);
    pd->show();
    analyseFile();
}


void HHistory::newHistogramType( int type )
{
    histType = type;
    pushButton9->setEnabled(TRUE);
    if(histStatus)
	delete histStatus;
    histStatus = new int[nFiles];
    for(int i=0;i<nFiles;i++)
	histStatus[i] = LowQAFile::GOOD;
}


void HHistory::stopScan( void )
{
    doScan = FALSE;
}


void HHistory::analyseFile( void )
{
    int i=1;
    LowQAFile *file;
    float mean;
    float error;
    
    for(file = files.first(); file && doScan; file=files.next())
    {
	qApp->processEvents(3);
	pd->setProgress(i);
	if(histType == 1)
	{
	    file->getMeanValue(histogram, path,&mean,&error);
	}
	else
	    file->getMeanValueTrend(histogram, path,&mean,&error);
	if((mean!=0)&&(error!=0))
	    meanHistogram->SetBinContent(i,mean);
	meanHistogram->SetBinError(i, 0. );
	i++;
    }
    pd->setProgress(files.count());
    delete pd;
    emit updateCutHists();
}


void HHistory::newCutHists( void )
{
    double min,max;
    Double_t Mp1S;
    Double_t Mm1S;
    Double_t Mp2S;
    Double_t Mm2S;
    
    meanOfHists = 0;
    sigma2Hmin = 0;    
    for(int i=0; i < nFiles;i++)
    {
	meanOfHists += meanHistogram->GetBinContent(i+1);
    }
    meanOfHists=meanOfHists/nFiles;
    if(meanValueLine)
	delete meanValueLine;
    meanValueLine =  new TH1F("meanValueLine","meanValueLine",1,0, nFiles);
    meanValueLine->SetBinContent(1,meanOfHists);
    meanValueLine->SetLineColor(2);

    for(Int_t i=0; i<nFiles; i++)
    {
	sigma2Hmin += (meanOfHists - meanHistogram->GetBinContent(i+1)) *
		      (meanOfHists - meanHistogram->GetBinContent(i+1));
    }
    sigma2Hmin = TMath::Sqrt( sigma2Hmin/Double_t(nFiles) );
    
    if(!asymquality)
    {
	Mp1S = meanOfHists+sigma2Hmin*qualityHighGood;
	Mm1S = meanOfHists-sigma2Hmin*qualityHighGood;
	Mp2S = meanOfHists+sigma2Hmin*qualityHighWarn;
	Mm2S = meanOfHists-sigma2Hmin*qualityHighWarn;
    }
    else
    {
	Mp1S = meanOfHists+sigma2Hmin*qualityHighGood;
	Mm1S = meanOfHists-sigma2Hmin*qualityLowGood;
	Mp2S = meanOfHists+sigma2Hmin*qualityHighWarn;
	Mm2S = meanOfHists-sigma2Hmin*qualityLowWarn;
    }
    if(hMeanPlus_1Sigma)
	delete hMeanPlus_1Sigma;
    hMeanPlus_1Sigma = new TH1F("hMeanPlus_1Sigma","hMeanPlus_1Sigma",1,0.,nFiles);
    if(hMeanMinus_1Sigma)
	delete hMeanMinus_1Sigma;
    hMeanMinus_1Sigma = new TH1F("hMeanMinus1Sigma","hMeanMinus1Sigma",1,0.,nFiles);
    if(hMeanPlus_2Sigma)
	delete hMeanPlus_2Sigma;
    hMeanPlus_2Sigma = new TH1F("hMeanPlus_2Sigma","hMeanPlus_2Sigma",1,0.,nFiles);
    if(hMeanMinus_2Sigma)
	delete hMeanMinus_2Sigma;
    hMeanMinus_2Sigma = new TH1F("hMeanMinus2Sigma","hMeanMinus2Sigma",1,0.,nFiles);
    
    if(!asymquality)
    {
	hMeanPlus_1Sigma->SetBinContent(1, meanOfHists+sigma2Hmin*qualityHighGood);
	hMeanMinus_1Sigma->SetBinContent(1, meanOfHists-sigma2Hmin*qualityHighGood);
	hMeanPlus_2Sigma->SetBinContent(1, meanOfHists+sigma2Hmin*qualityHighWarn);
	hMeanMinus_2Sigma->SetBinContent(1, meanOfHists-sigma2Hmin*qualityHighWarn);
    }
    else
    {
	hMeanPlus_1Sigma->SetBinContent(1, meanOfHists+sigma2Hmin*qualityHighGood);
	hMeanMinus_1Sigma->SetBinContent(1, meanOfHists-sigma2Hmin*qualityLowGood);
	hMeanPlus_2Sigma->SetBinContent(1, meanOfHists+sigma2Hmin*qualityHighWarn);
	hMeanMinus_2Sigma->SetBinContent(1, meanOfHists-sigma2Hmin*qualityLowWarn);
    }
    min = meanHistogram->GetBinContent(meanHistogram->GetMinimumBin());
    max = meanHistogram->GetBinContent(meanHistogram->GetMaximumBin());
    if(min>Mm2S){ min = Mm2S; }
    if(max<Mp2S){ max = Mp2S; }
    min = min-(max-min)*0.1;
    max = max+(max-min)*0.1;
    
    meanHistogram->SetMinimum(min);
    meanHistogram->SetMaximum(max);
    
    meanHistogram->Draw("E");
    if(Mm2S>0.)
    {
	hMeanPlus_2Sigma->SetFillColor(67);
	hMeanPlus_2Sigma->DrawCopy("SAME");
	
	hMeanPlus_1Sigma->SetFillColor(3); 
	hMeanPlus_1Sigma->DrawCopy("SAME");
	
	hMeanMinus_1Sigma->SetFillColor(67);
	hMeanMinus_1Sigma->DrawCopy("SAME");
	
	hMeanMinus_2Sigma->SetFillColor(196);
	hMeanMinus_2Sigma->DrawCopy("SAME");
    }
    if(Mm1S>0. && Mm2S<0.)
    {
	hMeanPlus_2Sigma->SetFillColor(67);
	hMeanPlus_2Sigma->DrawCopy("SAME");

	hMeanPlus_1Sigma->SetFillColor(3); 
	hMeanPlus_1Sigma->DrawCopy("SAME");

	hMeanMinus_1Sigma->SetFillColor(67);
	hMeanMinus_1Sigma->DrawCopy("SAME");

	hMeanMinus_2Sigma->SetFillColor(67);
	hMeanMinus_2Sigma->DrawCopy("SAME");
    }
    if(Mp1S>0. && Mm1S<0.)
    {
	hMeanPlus_2Sigma->SetFillColor(67);
	hMeanPlus_2Sigma->DrawCopy("SAME");

	hMeanPlus_1Sigma->SetFillColor(3); 
	hMeanPlus_1Sigma->DrawCopy("SAME");
	
	hMeanMinus_2Sigma->SetFillColor(67);
	hMeanMinus_2Sigma->DrawCopy("SAME");
	
	hMeanMinus_1Sigma->SetFillColor(3);
	hMeanMinus_1Sigma->DrawCopy("SAME");
    }
   if(Mp2S>0. && Mp1S<0.)
    {
	hMeanPlus_2Sigma->SetFillColor(67);
	hMeanPlus_2Sigma->DrawCopy("SAME");
	
	hMeanMinus_2Sigma->SetFillColor(67);
	hMeanMinus_2Sigma->DrawCopy("SAME");
	
	hMeanMinus_1Sigma->SetFillColor(3);
	hMeanMinus_1Sigma->DrawCopy("SAME");
	
	hMeanPlus_1Sigma->SetFillColor(67); 
	hMeanPlus_1Sigma->DrawCopy("SAME");
    }
    if(Mp2S<0.)
   {
	hMeanMinus_2Sigma->SetFillColor(67);
	hMeanMinus_2Sigma->DrawCopy("SAME");
	
	hMeanMinus_1Sigma->SetFillColor(3);
	hMeanMinus_1Sigma->DrawCopy("SAME");
	
	hMeanPlus_1Sigma->SetFillColor(67); 
	hMeanPlus_1Sigma->DrawCopy("SAME");
	
	hMeanPlus_2Sigma->SetFillColor(196);
	hMeanPlus_2Sigma->DrawCopy("SAME");
   }
    meanHistogram->Draw("SAME");
    meanValueLine->Draw("SAME");
    Canvas2->Update();
    pushButton49->setEnabled(TRUE);
    pushButton20_2->setEnabled(TRUE);
    pushButton10->setEnabled(TRUE);
}


void HHistory::printMeanHistogram( void )
{
   TPostScript *file;
    QString pname;
    
    printer = new QPrinter();
    if(printer->setup())
    {
	psName = new char[200];
	strcpy(psName,"/tmp/meanhist.XXXXXX");
	psName = mktemp(psName);
	file = new TPostScript(psName,111);
	Canvas2->cd();
	newCutHists();
	file->Close();
	pname = printer->printerName();
	lpr = new QProcess(this,"lpr");
	lpr->addArgument(QString("/usr/local/bin/lpr"));
	lpr->addArgument(QString("-P"));
	lpr->addArgument(pname.latin1());
	lpr->addArgument(QString(psName));
	connect(lpr, SIGNAL(processExited(void)),this, SLOT(printingDone(void)));
	lpr->start();
    }
}

void HHistory::saveMeanHistogram( void )
{
    QString fname = QFileDialog::getSaveFileName("./","Images( *.gif,  *.svg, *.pdf, *.eps )",this,"Save as...","Chose a filename to save histogram under...");
    if(fname)
    {
	Canvas2->getCanvas()->SaveAs(fname.latin1());
    }
}


void HHistory::printingDone( void )
{
    QString *msg;
    QFile *file;
    
    if(!lpr->normalExit())
    {
	if(lpr->exitStatus() != 0)
	{
	    msg = new QString("Error while printing on ");
	    msg->append(printer->printerName());
	    msg->append("\n");
	    while(lpr->canReadLineStderr())
	    {
		msg->append(lpr->readLineStderr());
	    }
	    QMessageBox::warning(this,"Printing failed",*msg,"Ok");
	}
    }
    else
    {
	msg = new QString("Succesfully printed image on ");
	msg->append(printer->printerName());
	msg->append("\n");
	while(lpr->canReadLineStdout())
	{
	    msg->append(lpr->readLineStdout());
	}
	QMessageBox::information(this,"Printing finished",*msg,"Ok");
    }
    delete lpr;
    file = new QFile(QString(psName));
    if(file->exists())
    {
	file->remove();
    }
}


void HHistory::enableInterval( void )
{
    if(checkBox3->isChecked())
    {
	pushButton14->setEnabled(TRUE);
    }
    else
    {
	pushButton14->setEnabled(FALSE);
    }
}


void HHistory::hideProgress( void )
{
    delete fileTimer;
    fileTimer = NULL;
    progressBar1->reset();
    textLabel1->setText("");
    pushButton24->setEnabled(TRUE);
    pushButton25->setEnabled(TRUE);
}


void HHistory::selectHistograms( void )
{
    bool ignorePrevSelection;
    int numOfValidFiles = 0;
    double x_dev2     = 0;
    double x_dev2GoodLow = 0;
    double x_dev2GoodHigh = 0;
    double x_dev2WarnLow = 0;
    double x_dev2WarnHigh = 0;
    int nGp=0, nWp=0, nBp=0;
    Int_t cntG=0, cntW=0, cntB=0;
    
    if(grG)
	delete grG;
    grG = new TGraph(1);
    if(grW)
	delete grW;
    grW = new TGraph(1);
    if(grB)
	delete grB;
    grB = new TGraph(1);
    ignorePrevSelection = checkBox4->isChecked();
    if(ignorePrevSelection)
    {
	for(int i=0;i<nFiles;i++)
	{
	    if(files.at(i)->getStatus((char *)histogram->latin1()) < LowQAFile::BAD)
	    {
		meanOfHists += meanHistogram->GetBinContent(i+1);
		numOfValidFiles++;
	    }
	}
	if(numOfValidFiles>0){
	    meanOfHists = meanOfHists/numOfValidFiles;
	}
	else
	{
	    meanOfHists = 0.;
	}
	for(int i=0;i<nFiles;i++)
	{
	    if(files.at(i)->getStatus((char *)histogram->latin1()) < LowQAFile::BAD)
	    {
		sigma2Hmin += (meanOfHists - meanHistogram->GetBinContent(i+1)) *
		      (meanOfHists - meanHistogram->GetBinContent(i+1));
	    }
	}
	if(numOfValidFiles>0)
	{
	    sigma2Hmin = TMath::Sqrt( sigma2Hmin/Double_t(nFiles) );
	}
	else
	{
	    sigma2Hmin = 1.0;
	}
    }
    if(asymquality)
    {
	x_dev2GoodLow = meanOfHists - (sigma2Hmin  * qualityLowGood);
	x_dev2GoodHigh = meanOfHists + (sigma2Hmin * qualityHighGood);
	x_dev2WarnLow = meanOfHists - (sigma2Hmin *  qualityLowWarn);
	x_dev2WarnHigh = meanOfHists + (sigma2Hmin * qualityHighWarn);
    }
    else
    {
	x_dev2GoodLow = meanOfHists - (sigma2Hmin * qualityHighGood);
	x_dev2GoodHigh = meanOfHists + (sigma2Hmin * qualityHighGood);
	x_dev2WarnLow = meanOfHists - (sigma2Hmin * qualityHighWarn);
	x_dev2WarnHigh = meanOfHists +(sigma2Hmin * qualityHighWarn);
    }
    for(int i=0;i<nFiles;i++)
    {
	x_dev2 = meanHistogram->GetBinContent(i+1);
	if((files.at(i)->getStatus((char *)histogram->latin1()) < LowQAFile::BAD) || ignorePrevSelection)
	{
	    if((x_dev2 > x_dev2WarnLow) && (x_dev2 < x_dev2WarnHigh))
	    {
		if(( x_dev2 > x_dev2GoodLow ) && (x_dev2 < x_dev2GoodHigh))
		{
		    nGp++;
		}	
		else
		{
		    nWp++;
		}
	    }
	    else
	    {
		nBp++;
	    }
	}
    }
    if(nGp>0)
    {
	grG->Set(nGp);
    }
    if(nWp>0)
    {
	grW->Set(nWp);
    }
    if(nBp>0)
    {
	grB->Set(nBp);
    }
    for(int i=0;i<nFiles;i++)
    {
	x_dev2 = meanHistogram->GetBinContent(i+1);;
	if((files.at(i)->getStatus((char *)(*path+*histogram).latin1()) < LowQAFile::BAD)  || ignorePrevSelection)
	{
	    if((x_dev2 > x_dev2WarnLow) && (x_dev2 < x_dev2WarnHigh))
	    {
		if(( x_dev2 > x_dev2GoodLow ) && (x_dev2 < x_dev2GoodHigh))
		{
		    if(nGp>0)
		    {		
			grG->SetPoint(cntG, Double_t(i+1)-0.5, Double_t(meanHistogram->GetBinContent(i+1)));
			cntG++;
			histStatus[i] = LowQAFile::GOOD;
		    }
		}
		else
		{
		    if(nWp>0)
		    {
			grW->SetPoint(cntW, Double_t(i+1)-0.5, Double_t(meanHistogram->GetBinContent(i+1)));
			cntW++;
			histStatus[i] = LowQAFile::WARN;
		    }
		}
	    }
	    else
	    {
		if(nBp>0)
		{
		    grB->SetPoint(cntB, Double_t(i+1)-0.5, Double_t(meanHistogram->GetBinContent(i+1)));
		    cntB++;
		    histStatus[i] = LowQAFile::BAD;
		}
	    }
	}
    }
    if(nGp>0)
    {
	grG->SetMarkerSize(0.5);
	grG->SetMarkerStyle(8);
	grG->SetMarkerColor(kBlack);
    }
    if(nWp>0)
    {
	grW->SetMarkerSize(0.5);
	grW->SetMarkerStyle(8);
	grW->SetMarkerColor(kBlue);
    }
    if(nBp>0)
    {
	grB->SetMarkerSize(0.5);
	grB->SetMarkerStyle(21);
	grB->SetMarkerColor(kRed);
    }
    Canvas2->cd();
   if(nBp>0)
    {
	grB->Draw("P");
    }
   if(nGp>0)
   {
	grG->Draw("P");
    }
   if(nWp>0)
   {
	grW->Draw("P");
    }
   Canvas2->Update();
   pushButton11->setEnabled(TRUE);
}


void HHistory::markHistograms( void )
{   
    for(int i=0;i<nFiles;i++)
    {
	files.at(i)->updateHistogramStatus((*path+*histogram),histStatus[i],TRUE);
    }
    emit updateFileList();
}


void HHistory::showHelp( void )
{
    HelpDialog *dlg;
    
    dlg = new HelpDialog(this, "Help");
    dlg->setHelpFile(*helpName);
    dlg->setTopic(tabWidget->currentPageIndex());
    dlg->exec();
}
